/*
SocketCommClass.h

Serial Comm functions
*/

#ifndef _SOCKETCOMMCLASS_H
#define _SOCKETCOMMCLASS_H

#include "Xpert/Win32Fix.h"
#include "Socket.h"

#undef DECL
#if defined(_AFXDLL) || defined(XPERT_EXE)
#ifdef UTILS_DLL
#define DECL _declspec(dllexport)
#else
#define DECL _declspec(dllimport)
#endif
#else
#define DECL 
#endif

class DECL CSocketComm
{
public:
   CSocketComm();
   ~CSocketComm();
   CSocketComm(const TCHAR* CommPort);
   CSocketComm& operator=(const CSocketComm& SC);

   // open/close
   bool CloseComm();
   bool OpenComm(CString UserName=_T(""), CString Password=_T(""));
   bool IsOpen() { return Client.IsConnected(); };

   // configuration
   void SetConfiguration(DWORD BaudRate, BYTE ByteSize, BYTE Parity, BYTE StopBits, BOOL FlowCtrl);
   void SetCommPort(const TCHAR* CommPort) {_tcscpy(m_CommPort, CommPort);};
   void SetBaudRate(const DWORD BaudRate) {m_BaudRate = BaudRate;};
   void SetTimeouts(COMMTIMEOUTS& CommTimeouts) {m_CommTimeouts = CommTimeouts;};
   // data
#ifdef UNICODE
   bool GetChar(char  * ch) {return GetChar((UCHAR*) ch);};
#endif
   bool GetChar(UCHAR * uch);
   bool GetChar(TCHAR * wch);

   int  GetStr(UCHAR * Str, int StrLen);
#ifdef UNICODE
   int  GetStr(char *  Str, int StrLen) {return GetStr((UCHAR*)Str, StrLen);};
#endif
   int  GetStr(TCHAR * Str, int StrLen);
   int Get(void* Buffer, int BufferLen);

   bool Put(void* Buffer, int BufferLen);
   bool PutAndWait(void* Buffer, int BufferLen);
#ifdef UNICODE
   bool PutChar(const char  ch) {return PutChar((const UCHAR)ch);};
#endif
   bool PutChar(const UCHAR uch);
   bool PutChar(const TCHAR wch);


#ifdef UNICODE
   bool PutStr(const char * Str) {return PutStr((UCHAR*)Str);};
   bool PutStr(const char * Str, int StrLen) {return PutStr((UCHAR*)Str, StrLen);};
   bool PutStrAndWait(const char * Str) {return PutStrAndWait((UCHAR*)Str);};
   bool PutStrAndWait(const char * Str, int StrLen) {return PutStrAndWait((UCHAR*)Str, StrLen);};
#endif
   bool PutStr(const UCHAR * Str) {return (PutStr(Str, strlen((char*)Str)));};
   bool PutStr(const UCHAR * Str, int StrLen);
   bool PutStr(TCHAR * Str) { return PutStr(Str, _tcsclen(Str));};
   bool PutStr(TCHAR * Str, int StrLen);
   bool PutStr(CString Str);

   bool PutStrAndWait(const UCHAR * Str) {return (PutStrAndWait(Str, strlen((char*)Str)));};
   bool PutStrAndWait(const UCHAR * Str, int StrLen);
   bool PutStrAndWait(TCHAR * Str) { return PutStrAndWait(Str, _tcsclen(Str));};
   bool PutStrAndWait(TCHAR * Str, int StrLen);
   bool PutStrAndWait(CString Str);

   int  NumberBytesInputBuffer(); // returns number of bytes in input buffer
   bool KeyPressed();             // return true if a key is in rx buffer
   bool KeyPressedTimeout(DWORD Timeout); // return true if a key is in rx buffer (waits up to Timeout ms)
   void FlushInput();             // empties rx buffer

   TSocketClient& GetClient() {return Client;};

   void WaitOnRx();                // waits (blocks) until a char comes in the serial port - flushes input first
   void WaitForTxEmpty();          // waits (blocks) until transmit buffer empty.
   void WaitForTxEmpty(TCHAR LastCh); // More accurate version of WaitForTxEmpty()

// SocketComm methods ...
   bool Read(LPVOID lpBuffer, DWORD nNumberOfBytesToRead, LPDWORD lpNumberOfBytesRead, DWORD Timeout);
   bool Read(TSSPPacket& Packet, DWORD Timeout);
   bool Write(LPVOID lpBuffer, DWORD nNumberOfBytesToWrite);
   bool WriteAndWait(LPVOID lpBuffer, DWORD nNumberOfBytesToRead);
   bool Write(TSSPPacket& Packet);
   bool WriteAndWait(TSSPPacket& Packet);
   bool SetCapture(bool State);
   void EscapeCommFunction(DWORD Func);
   bool GetCommModemStatus(LPDWORD lpModemStat);
   BOOL SetCommMask(DWORD dwEvtMask);
   BOOL WaitCommEvent(LPDWORD lpEvtMask, DWORD Timeout = INFINITE);
   BOOL GetCommMask(LPDWORD lpEvtMask);
   BOOL SetCommTimeouts(LPCOMMTIMEOUTS lpCommTimeouts);
   BOOL GetCommTimeouts(LPCOMMTIMEOUTS lpCommTimeouts);
   BOOL GetCommState(LPDCB lpDCB);
   BOOL SetCommState(LPDCB lpDCB);
   bool LockComm(DWORD Timeout);
   bool UnLockComm(bool NoHangup=false, bool bNoLogout=false);
   bool Logout(bool NoHangup=false);
   void SetHost(LPCTSTR NewHost);
   bool GetPortList(TSSPData& Reply, CString UserName=_T(""), CString Password=_T(""));
   bool GetComOptions(int& Options);              // See TComOptions in ssp.h
   bool SetComOptions(int Options, int Mask);     // See TComOptions in ssp.h
   bool SetExtendedCommands(CString CommandList);
   bool RunCommand(CString CommandLine);


protected:
   bool LowReadRaw(TSSPData& Data, DWORD StartTime, DWORD Now, DWORD Timeout, TProtoOpcode& Opcode, TProtoOpcode& DataMatch);
   bool LowWaitForAck(TProtoOpcode Match, DWORD Timeout);
   bool LowWaitForOpcode(TProtoOpcode Match, DWORD Timeout);
   bool LowWaitForData(DWORD Timeout);
   bool LowWaitForPacket(DWORD Timeout);
//
   bool SendAndWaitForAck(TSSPData& Data, DWORD Timeout);
   bool SendAndWaitForOpcode(TSSPData& Data, DWORD Timeout, TProtoOpcode Match);

protected:
   TCHAR   m_CommPort[8];
   DWORD   m_BaudRate;
   BOOL    m_HardwareFlow; // None/Hardware
   BYTE    m_ByteSize;     // number of bits per byte (4-8)
   BYTE    m_Parity;       // 0-4 = no,odd,even,mark,space
   BYTE    m_StopBits;     // 0,1,2 = 1, 1.5, 2
   
   COMMTIMEOUTS m_CommTimeouts;

protected:
   friend class TLockAccess;

   DCB LastDCB;
   TSocketClient Client;
   CRITICAL_SECTION WaitLock;
   CRITICAL_SECTION DataLock;
   TData LastData;
   TSSPPacket LastPacket;
   DWORD LastModemStatus;
   DWORD OldModemStatus;
   bool LastCapture;
   DWORD CommMask;
   bool InWaitCommEvent;
   CString Host;
   TComOptions LastOptions;
};

class TLockAccess
{
public:
   TLockAccess(CSocketComm& aComm);
   ~TLockAccess();
private:
   CSocketComm& Comm;
};


#endif
